package net.pharmatrak.babelj;

import java.io.*;
import java.net.*;

/**
 * <blockquote><i>
 * Babelj is a language server library for Java.
 * Copyright (C) 1999  Pharmatrak
 * <p>
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * <p>
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * <p>
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 * <p>
 * </i></blockquote>
 *
 * @author  Timothy W Macinta (twm@alum.mit.edu)
 **/

public class BabeljServer implements Runnable {

  public static final int DEFAULT_BABELJ_PORT = 9988;
  
  ServerSocket sock;
  boolean running = true;

  public BabeljServer(int port) throws IOException {
    this.sock = new ServerSocket(port);
  }

  public void run() {
    while (running) {
      Socket s = null;
      try {
	s = sock.accept();
	if (checkHostAddress(s.getInetAddress())) {
	  new BabeljConnection(s);
	} else {
	  System.err.println("Warning - Attempted connection from "+s.getInetAddress());
	  s.close();
	}
      } catch (IOException e) {
	e.printStackTrace();
	try {
	  if (s != null) s.close();
	} catch (IOException e2) {}
      }
    }
  }

  public void stop() {
    running = false;
  }

  boolean checkHostAddress(InetAddress addr) throws UnknownHostException {
    if (addr.getHostAddress().equals("127.0.0.1")) return true;
    return addr.equals(InetAddress.getLocalHost());
  }

  /** Usage:  java net.pharmatrak.babelj.BabeljServer [port] **/
  public static void main(String[] arg) {
    try {
      int port = DEFAULT_BABELJ_PORT;
      if (arg.length > 0) port = Integer.parseInt(arg[0]);
      new BabeljServer(port).run();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

}
