package net.pharmatrak.babelj.test;

import java.io.*;
import java.net.*;

/**
 * <blockquote><i>
 * Babelj is a language server library for Java.
 * Copyright (C) 1999  Pharmatrak
 * <p>
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * <p>
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * <p>
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 * <p>
 * </i></blockquote>
 *
 * OK, this class seems pointless - why write a Babelj client
 * in Java?  Well it's for testing so calm down.  It's also
 * an illustration of how to write a client.
 * <p>
 * Unimplemented commands:  none!
 *
 * @author  Timothy W Macinta (twm@alum.mit.edu)
 **/

public class BabeljJavaClient {

  String last_type_template = "";
  Socket sock;
  InputStream in;
  OutputStream out;

  public BabeljJavaClient() {
  }

  public void connect() throws IOException {
    connect(9988);
  }

  public void connect(int port) throws IOException {
    this.sock = new Socket("127.0.0.1", port);
    this.in = new BufferedInputStream(sock.getInputStream());
    this.out = new BufferedOutputStream(sock.getOutputStream());
    writeString("babelj_version\nBabelj Version 0.1\n");
    flush();
    readObject();
  }

  public void disconnect() throws IOException {
    writeString("logout\n");
    flush();
    try {
      readObject();
      sock.close();
    } catch (IOException e) {}
  }

  public Object methodCall(Object obj_ref, String method_name, Object[] args) throws IOException {
    return methodCall(obj_ref, method_name, args, null);
  }

  public Object methodCall(Object obj_ref, String method_name, Object[] args, Object[] type_template) throws IOException {
    writeString("method_call\n");
    writeObject(obj_ref, "ref");
    writeObject(method_name, "string");
    writeObject(args, type_template);
    flush();
    return readObject();
  }

  public Object staticMethodCall(String class_name, String method_name, Object[] args) throws IOException {
    return staticMethodCall(class_name, method_name, args, null);
  }

  public Object staticMethodCall(String class_name, String method_name, Object[] args, Object[] type_template) throws IOException {
    writeString("static_method_call\n");
    writeObject(class_name, "string");
    writeObject(method_name, "string");
    writeObject(args, type_template);
    flush();
    return readObject();
  }

  public Object newInstanceCall(String class_name, Object[] args) throws IOException {
    return newInstanceCall(class_name, args, null);
  }

  public Object newInstanceCall(String class_name, Object[] args, Object[] type_template) throws IOException {
    writeString("new_instance\n");
    writeObject(class_name, "string");
    writeObject(args, type_template);
    flush();
    return readObject();
  }

  public Object getStaticField(String class_name, String field_name) throws IOException {
    writeString("get_static_field\n");
    writeObject(class_name, "string");
    writeObject(field_name, "string");
    flush();
    return readObject();
  }
  
  public Object getField(Object obj_ref, String field_name) throws IOException {
    writeString("get_field\n");
    writeObject(obj_ref, "ref");
    writeObject(field_name, "string");
    flush();
    return readObject();
  }
  
  public Object freeRef(Object obj_ref) throws IOException {
    writeString("free_ref\n");
    writeObject(obj_ref, "ref");
    flush();
    return readObject();
  }
  
  public boolean lastWasException() {
    return last_type_template.equals("exception");
  }

  public boolean lastWasNull() {
    return last_type_template.equals("null");
  }

  String getLastTypeTemplate() {
    return last_type_template;
  }



  
  ///
  ///
  /// End of User Callable Functions.
  /// Please do not use the rest of these
  /// functions in external code as they
  /// are not part of the API and are
  /// subject to change.
  ///
  ///

  
  public void writeObject(Object obj, Object type_template) throws IOException {

    // recursively write arrays

    if (obj instanceof Object[]) {
      Object[] ar = (Object[]) obj;
      writeString("array\n");
      writeString(ar.length+"\n");
      Object[] ta = null;
      if (type_template != null && type_template instanceof Object[]) ta = (Object[]) type_template;
      for (int i = 0; i < ar.length; i++) {
	if (ta != null) {
	  writeObject(ar[i], ta[i]);
	} else {
	  writeObject(ar[i], type_template);
	}
      }
      return;
    }

    // write non-arrays

    if (type_template == null) {
      
      // This type could actually be detected pretty well.  Maybe will do later.

      type_template = "var";
    }
    writeString(type_template.toString());
    writeString("\n");
    if (type_template.equals("string") || type_template.equals("var")) {
      writeString(obj.toString().length()+"\n");
      writeString(obj.toString());
      return;
    } else if (type_template.equals("null")) {
      return;
    } else {
      writeString(obj.toString());
      writeString("\n");
      return;
    }
  }

  Object readObject() throws IOException {
    String type_template = readString();
    this.last_type_template = type_template;

    // catch exceptions

    if (type_template.equals("exception")) {
      Object explain = readObject();
      System.err.println(explain.toString());
      return explain;
    }

    // recursively read arrays

    if (type_template.equals("array")) {
      int ar_size = Integer.parseInt(readString());
      Object a[] = new Object[ar_size];
      for (int i = 0; i < ar_size; i++) {
	a[i] = readObject();
      }
      return a;
    }

    // read non-arrays

    if (type_template.equals("string") || type_template.equals("var")) {
      int len = Integer.parseInt(readString());
      return readString(len);
    } else if (type_template.equals("boolean")) {
      String obj = readString();
      if (obj.equals("true")) return Boolean.TRUE;
      else return Boolean.FALSE;
    } else if (type_template.equals("null")) {
      return null;
    } else { // probably shouldn't lump all the numbers in here
      return readString();
    }
  }

  String readString() throws IOException {
    int c;
    StringBuffer sb = new StringBuffer();
    while ((c = in.read()) != '\n') {
      if (c < 0) throw new EOFException();
      sb.append((char) c);
    }
    return new String(sb);
  }

  String readString(int strlen) throws IOException {
    int c = 0;
    StringBuffer sb = new StringBuffer();
    while (strlen-- > 0 && (c = in.read()) >= 0) {
      sb.append((char) c);
    }
    if (c < 0) throw new EOFException();
    return new String(sb);
  }

  void writeString(String str) throws IOException {
    out.write(str.getBytes());
  }

  void flush() throws IOException {
    out.flush();
  }

}
